﻿using System;
using System.Collections.Generic;
using HAServer.Interfaces.Core;
using InfluxDB.LineProtocol.Client;
using InfluxDB.LineProtocol.Payload;

namespace HAServer.InfluxDB {
	public class InfluxDB:IComponent, ISubscriber {
		private readonly IMessageBus _messageBus;
		private readonly ISettingsProvider _settingsProvider;
		private readonly ILogProvider _logProvider;
		private readonly LineProtocolClient _client;
		private readonly Queue<LineProtocolPoint> _dataQueue;

		public InfluxDB(IMessageBus messageBus, ISettingsProvider settingsProvider, ILogProvider logProvider) {
			_messageBus = messageBus;
			_settingsProvider = settingsProvider;
			_logProvider = logProvider;
			var uri = $"http://{_settingsProvider.Host()}:{settingsProvider.Port()}";
			_client = new LineProtocolClient(new Uri(uri),
											 _settingsProvider.Database(),
											 _settingsProvider.Username(),
											 _settingsProvider.Password());
			_dataQueue = new Queue<LineProtocolPoint>();
		}

		public void Initialise() {
			_messageBus.AddSubscriber(this, _settingsProvider.Topic());
		}

		public void Shutdown() {
		}

		private void QueueData(string topic, string message) {
			double.TryParse(message, out var numeric);
			var data = new LineProtocolPoint(_settingsProvider.Measurement(),
			                                 new Dictionary<string, object> {{"value", message}, {"valueNumeric", numeric}},
			                                 new Dictionary<string, string> {{"topic", topic}},
			                                 DateTime.UtcNow);
			_dataQueue.Enqueue(data);
		}


		private void SendData() {
			var payload = new LineProtocolPayload();

			foreach (var data in _dataQueue)
				payload.Add(data);

			try {
				var result = _client.WriteAsync(payload).Result;

				if (result.Success)
					_dataQueue.Clear();
				else
					_logProvider.Write(result.ErrorMessage, LogLevel.Error);
			}
			catch (Exception e) {
				_logProvider.Write(e.ToString(), LogLevel.Error);
			}
		}

		public void ProcessMessage(string topic, string message) {
			QueueData(topic, message);
			SendData();
		}
	}
}
