﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.Text;
using HAServer.Interfaces.Core;
using uPLibrary.Networking.M2Mqtt;
using uPLibrary.Networking.M2Mqtt.Messages;

namespace HAServer.MQTT {
	[ExcludeFromCodeCoverage]
	public class MqttManager : IComponent, ISubscriber {
		private readonly MqttClient _client;
		private readonly IMessageBus _messageBus;
		private readonly bool _retain;
		private readonly byte _qosLevel;

		public MqttManager(IMessageBus messageBus, ISettingsProvider settings) {
			_messageBus = messageBus;
			_qosLevel = settings.MqttQos();
			_retain = settings.MqttRetain();
			_messageBus.AddSubscriber(this, $"{settings.MqttBaseTopic()}/.*");

			_client = new MqttClient(settings.MqttHost());
			_client.MqttMsgPublishReceived += MessageRecieved;
			_client.Subscribe(new[] { $"{settings.MqttBaseTopic()}/#" }, new[] { _qosLevel });
		}

		void MessageRecieved(object sender, MqttMsgPublishEventArgs e) {
			var message = Encoding.UTF8.GetString(e.Message);
			_messageBus.PublishMessage(e.Topic, message, this);
		}

		public void ProcessMessage(string topic, string message) {
			_client.Publish(topic, Encoding.UTF8.GetBytes(message), _qosLevel, _retain);
		}

		public void Initialise() {
			_client.Connect(Environment.MachineName);
		}

		public void Shutdown() {
			_client.Disconnect();
		}
	}
}
