﻿using System;
using HAServer.Components;
using HAServer.Core;
using HAServer.Devices;
using HAServer.Interfaces.Core;
using Moq;
using NUnit.Framework;

namespace HAServer.Tests.Integration {
	[TestFixture]
	public class IntegrationTests {
		private const string SolarEventsJson = @"{""results"":{""sunrise"":""2017-05-29T03:49:06+00:00"",""sunset"":""2017-05-29T20:22:45+00:00"",""solar_noon"":""2017-05-29T12:05:55+00:00"",""day_length"":59619,""civil_twilight_begin"":""2017-05-29T03:00:26+00:00"",""civil_twilight_end"":""2017-05-29T21:11:25+00:00"",""nautical_twilight_begin"":""2017-05-29T01:43:51+00:00"",""nautical_twilight_end"":""2017-05-29T22:28:00+00:00"",""astronomical_twilight_begin"":""1970-01-01T00:00:01+00:00"",""astronomical_twilight_end"":""1970-01-01T00:00:01+00:00""},""status"":""OK""}";

		[Test]
		public void WhenTheSunSets_TurnSwitchOff() {
			var logProvider=new Mock<ILogProvider>();
			var bus = new MessageBus(logProvider.Object);

			var settingsProvider = new Mock<ISettingsProvider>();
			settingsProvider.Setup(provider => provider.GetValue(It.IsAny<string>(), It.IsAny<string>())).Returns("0");

			var mockApi = new Mock<IFileGet>();
			mockApi.Setup(get => get.Get(It.IsAny<string>())).Returns(SolarEventsJson);
			var solarEvents = new SolarEvents(bus, settingsProvider.Object, mockApi.Object, logProvider.Object, TimeZoneInfo.Utc);
			solarEvents.Initialise();

			var devices = new DeviceConfig();
			var @switch = new Switch(bus, "switch", "Switch", "1", "0") { On = true };
			devices.AddDevice(@switch);

			var conditions = new[] {new Condition(Comparator.Equal, "solar_event", "civil_twilight_end")};
			var actions = new[] {new TriggerAction(TriggerActionType.Device, @switch.Topic, @switch.TurnOff)};
			var trigger = new Trigger(bus, "trigger", TriggerType.Any, actions, conditions, logProvider.Object);

			bus.PublishMessage("time", "21:11:25", null);
			Assert.That(@switch.On, Is.False);
		}

		[Test]
		public void WhenInputIsLessThanSensor_TurnSwitchOn() {
			var logProvider=new Mock<ILogProvider>();
			var bus = new MessageBus(logProvider.Object);

			var settingsProvider = new Mock<ISettingsProvider>();
			settingsProvider.Setup(provider => provider.GetValue(It.IsAny<string>(), It.IsAny<string>())).Returns("0");

			var mockApi = new Mock<IFileGet>();
			mockApi.Setup(get => get.Get(It.IsAny<string>())).Returns(SolarEventsJson);
			var solarEvents = new SolarEvents(bus, settingsProvider.Object, mockApi.Object, logProvider.Object, TimeZoneInfo.Utc);
			solarEvents.Initialise();

			var devices = new DeviceConfig();
			var @switch = new Switch(bus, "switch", "Switch", "1", "0");
			var sensor = new Sensor(bus, "sensor", "sensor", "");
			var input = new Sensor(bus, "input", "input", "");
			devices.AddDevice(@switch);
			devices.AddDevice(sensor);
			devices.AddDevice(input);

			var conditions2 = new[] { new DeviceCondition(Comparator.LessThan, "sensor", input)};
			var actions = new[] { new TriggerAction(TriggerActionType.Device, @switch.Topic, @switch.TurnOn) };
			var trigger2 = new Trigger(bus, "trigger2", TriggerType.Any, actions, conditions2, logProvider.Object);

			bus.PublishMessage("input", "2", null);
			bus.PublishMessage("sensor", "1", null);
			Assert.That(@switch.On, Is.True);
		}
	}
}
