﻿using System;
using FluentAssertions;
using HAServer.Core;
using Xunit;

namespace HAServer.Tests.Core {
	public class ConditionTests {

		/*------------------------+
		| String comparison tests |
		+----------------------- */

		[Fact]
		public void WhenConditionIsEqual_WithStringInputThatIsEqual_ReturnTrue() {
			var condition=new Condition(Comparator.Equal, "", "test");
			condition.IsTrue("test").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsEqual_WithStringInputThatIsNotEqual_ReturnFalse() {
			var condition = new Condition(Comparator.Equal, "", "test");
			condition.IsTrue("test2").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsNotEqual_WithStringInputThatIsEqual_ReturnTrue() {
			var condition = new Condition(Comparator.NotEqual, "", "test");
			condition.IsTrue("test2").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsNotEqual_WithStringInputThatIsNotEqual_ReturnFalse() {
			var condition = new Condition(Comparator.NotEqual, "", "test");
			condition.IsTrue("test").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThan_WithAnyStringInput_ReturnFalse() {
			var condition = new Condition(Comparator.GreaterThan, "", "test");
			condition.IsTrue("test").Should().BeFalse();
			condition.IsTrue("test2").Should().BeFalse();
			condition.IsTrue("tes").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsLessThan_WithAnyStringInput_ReturnFalse() {
			var condition = new Condition(Comparator.LessThan, "", "test");
			condition.IsTrue("test").Should().BeFalse();
			condition.IsTrue("test2").Should().BeFalse();
			condition.IsTrue("tes").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThanOrEqual_WithAnyStringInput_ReturnFalse() {
			var condition = new Condition(Comparator.GreaterThanOrEqual, "", "test");
			condition.IsTrue("test").Should().BeFalse();
			condition.IsTrue("test2").Should().BeFalse();
			condition.IsTrue("tes").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsLessThanOrEqual_WithAnyStringInput_ReturnFalse() {
			var condition = new Condition(Comparator.LessThanOrEqual, "", "test");
			condition.IsTrue("test").Should().BeFalse();
			condition.IsTrue("test2").Should().BeFalse();
			condition.IsTrue("tes").Should().BeFalse();
		}

		/*------------------------+
		| Number comparison tests |
		+------------------------*/

		[Fact]
		public void WhenConditionIsEqual_WithNumericInputThatIsEqual_ReturnTrue() {
			var condition = new Condition(Comparator.Equal, "", "1");
			condition.IsTrue("1").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsEqual_WithNumericInputThatIsNotEqual_ReturnFalse() {
			var condition = new Condition(Comparator.Equal, "", "1");
			condition.IsTrue("2").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsEqual_WithDecimalInputThatIsEqual_ReturnTrue() {
			var condition = new Condition(Comparator.Equal, "", "1.123");
			condition.IsTrue("1.123").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsEqual_WithDecimalInputThatIsNotEqual_ReturnFalse() {
			var condition = new Condition(Comparator.Equal, "", "1.123");
			condition.IsTrue("2.123").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsNotEqual_WithNumericInputThatIsNotEqual_ReturnTrue() {
			var condition = new Condition(Comparator.NotEqual, "", "1");
			condition.IsTrue("2").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsNotEqual_WithNumericInputThatIsEqual_ReturnFalse() {
			var condition = new Condition(Comparator.NotEqual, "", "1");
			condition.IsTrue("1").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThan_WithLargerNumericInput_ReturnTrue() {
			var condition = new Condition(Comparator.GreaterThan, "", "1");
			condition.IsTrue("2").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsGreaterThan_WithSmallerNumericInput_ReturnFalse() {
			var condition = new Condition(Comparator.GreaterThan, "", "1");
			condition.IsTrue("0").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThan_WithEqualNumericInput_ReturnFalse() {
			var condition = new Condition(Comparator.GreaterThan, "", "1");
			condition.IsTrue("1").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThanOrEqual_WithLargerNumericInput_ReturnTrue() {
			var condition = new Condition(Comparator.GreaterThanOrEqual, "", "1");
			condition.IsTrue("2").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsGreaterThanOrEqual_WithSmallerNumericInput_ReturnFalse() {
			var condition = new Condition(Comparator.GreaterThanOrEqual, "", "1");
			condition.IsTrue("0").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThanOrEqual_WithEqualNumericInput_ReturnTrue() {
			var condition = new Condition(Comparator.GreaterThanOrEqual, "", "1");
			condition.IsTrue("1").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsLessThan_WithLargerNumericInput_ReturnFalse() {
			var condition = new Condition(Comparator.LessThan, "", "1");
			condition.IsTrue("2").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsLessThan_WithSmallerNumericInput_ReturnTrue() {
			var condition = new Condition(Comparator.LessThan, "", "1");
			condition.IsTrue("0").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsLessThan_WithEqualNumericInput_ReturnFalse() {
			var condition = new Condition(Comparator.LessThan, "", "1");
			condition.IsTrue("1").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsLessThanOrEqual_WithLargerNumericInput_ReturnFalse() {
			var condition = new Condition(Comparator.LessThanOrEqual, "", "1");
			condition.IsTrue("2").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsLessThanOrEqual_WithSmallerNumericInput_ReturnTrue() {
			var condition = new Condition(Comparator.LessThanOrEqual, "", "1");
			condition.IsTrue("0").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsLessThanOrEqual_WithEqualNumericInput_ReturnTrue() {
			var condition = new Condition(Comparator.LessThanOrEqual, "", "1");
			condition.IsTrue("1").Should().BeTrue();
		}

		/*----------------------+
		| Time comparison tests |
		+----------------------*/

		[Fact]
		public void WhenConditionIsEqual_WithTimeInputThatIsEqual_ReturnTrue() {
			var condition = new Condition(Comparator.Equal, "", "1:00");
			condition.IsTrue("1:00").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsEqual_WithTimeInputThatIsNotEqual_ReturnFalse() {
			var condition = new Condition(Comparator.Equal, "", "1:00");
			condition.IsTrue("2:00").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsNotEqual_WithTimeInputThatIsNotEqual_ReturnTrue() {
			var condition = new Condition(Comparator.NotEqual, "", "1:00");
			condition.IsTrue("2:00").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsNotEqual_WithTimeInputThatIsEqual_ReturnFalse() {
			var condition = new Condition(Comparator.NotEqual, "", "1:00");
			condition.IsTrue("1:00").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThan_WithLargerTimeInput_ReturnTrue() {
			var condition = new Condition(Comparator.GreaterThan, "", "1:00");
			condition.IsTrue("2:00").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsGreaterThan_WithSmallerTimeInput_ReturnFalse() {
			var condition = new Condition(Comparator.GreaterThan, "", "1:00");
			condition.IsTrue("0:00").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThan_WithEqualTimeInput_ReturnFalse() {
			var condition = new Condition(Comparator.GreaterThan, "", "1:00");
			condition.IsTrue("1:00").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThanOrEqual_WithLargerTimeInput_ReturnTrue() {
			var condition = new Condition(Comparator.GreaterThanOrEqual, "", "1:00");
			condition.IsTrue("2:00").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsGreaterThanOrEqual_WithSmallerTimeInput_ReturnFalse() {
			var condition = new Condition(Comparator.GreaterThanOrEqual, "", "1:00");
			condition.IsTrue("0:00").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsGreaterThanOrEqual_WithEqualTimeInput_ReturnTrue() {
			var condition = new Condition(Comparator.GreaterThanOrEqual, "", "1:00");
			condition.IsTrue("1:00").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsLessThan_WithLargerTimeInput_ReturnFalse() {
			var condition = new Condition(Comparator.LessThan, "", "1:00");
			condition.IsTrue("2:00").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsLessThan_WithSmallerTimeInput_ReturnTrue() {
			var condition = new Condition(Comparator.LessThan, "", "1:00");
			condition.IsTrue("0:00").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsLessThan_WithEqualTimeInput_ReturnFalse() {
			var condition = new Condition(Comparator.LessThan, "", "1:00");
			condition.IsTrue("1:00").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsLessThanOrEqual_WithLargerTimeInput_ReturnFalse() {
			var condition = new Condition(Comparator.LessThanOrEqual, "", "1:00");
			condition.IsTrue("2:00").Should().BeFalse();
		}

		[Fact]
		public void WhenConditionIsLessThanOrEqual_WithSmallerTimeInput_ReturnTrue() {
			var condition = new Condition(Comparator.LessThanOrEqual, "", "1:00");
			condition.IsTrue("0:00").Should().BeTrue();
		}

		[Fact]
		public void WhenConditionIsLessThanOrEqual_WithEqualTimeInput_ReturnTrue() {
			var condition = new Condition(Comparator.LessThanOrEqual, "", "1:00");
			condition.IsTrue("1:00").Should().BeTrue();
		}

		/*-----------------------+
		| Other comparison tests |
		+-----------------------*/

		[Fact]
		public void WhenUsingInvalidComparator_ThrowsException() {
			var condition = new Condition((Comparator)10, "", "1:00");
			Func<bool> func = () => condition.IsTrue("1:00");
			func.Should().Throw<ArgumentOutOfRangeException>();
		}
	}
}
