﻿using FluentAssertions;
using HAServer.Core;
using HAServer.Devices;
using HAServer.Interfaces.Core;
using NSubstitute;
using Xunit;

namespace HAServer.Tests.Core {
	public class DeviceGroupTests {
		[Fact]
		public void AddDevice_WithValidDevice_AccessorReturnsDevice() {
			var bus = Substitute.For<IMessageBus>();
			var device = Substitute.For<BaseDevice>(bus, string.Empty, string.Empty);
			var group = new DeviceGroup(string.Empty);
			group.Add(device);
			group[0].Should().Be(device);
		}

		[Fact]
		public void RemoveDevice_WithValidDevice_CollectionIsEmpty() {
			var bus = Substitute.For<IMessageBus>();
			var device = Substitute.For<BaseDevice>(bus, string.Empty, string.Empty);
			var group = new DeviceGroup(string.Empty);
			group.Add(device);
			group.Remove(device);
			group.Should().BeEmpty();
		}

		[Fact]
		public void TurnOn_TurnsOnAllSwitches() {
			var bus = Substitute.For<IMessageBus>();
			var device = Substitute.For<Switch>(bus, "switch", "switch", "1", "0");

			var group = new DeviceGroup(string.Empty);
			group.Add(device);
			group.TurnOn();

			device.On.Should().BeTrue();
		}

		[Fact]
		public void TurnOff_TurnsOffAllSwitches() {
			var bus = Substitute.For<IMessageBus>();
			var device = Substitute.For<Switch>(bus, "switch", "switch", "1", "0");
			device.On = true;

			var group = new DeviceGroup(string.Empty);
			group.Add(device);
			group.TurnOff();

			device.On.Should().BeFalse();
		}
	}
}
