﻿using System;
using System.IO;
using FluentAssertions;
using HAServer.Core;
using HAServer.Interfaces.Core;
using Xunit;

namespace HAServer.Tests.Core {
	public class LogTests {
		[Fact]
		public void Write_WithLogLevelLessThanSet_WritesToLog() {
			var logFileLocation = Path.Combine(Useful.GetTestDirectory(), "test.txt");
			var log = new Log(logFileLocation, LogLevel.Warning, LogLevel.Warning);
			if (File.Exists(log.FileLocation))
				File.Delete(log.FileLocation);
			log.Write("test", LogLevel.Error);
			var expected = $"{LogLevel.Error} - {DateTime.Now} - test{Environment.NewLine}";
			var text = File.ReadAllText(log.FileLocation);
			text.Should().Be(expected);
		}

		[Fact]
		public void Write_WithLogLevelGreaterThanSet_DoesNotWriteToLog() {
			var logFileLocation = Path.Combine(Useful.GetTestDirectory(), "test.txt");
			var log = new Log(logFileLocation, LogLevel.Error, LogLevel.Error);
			if (File.Exists(log.FileLocation))
				File.Delete(log.FileLocation);
			log.Write("test", LogLevel.Warning);
			File.Exists(log.FileLocation).Should().BeFalse();
		}
	}
}
