﻿using System.Configuration;
using System.Diagnostics.CodeAnalysis;

namespace HAServer.Core {
	[ExcludeFromCodeCoverage]
	public class ApplicationSettingsProvider : IApplicationSettingsProvider {
		private readonly Configuration _config;

		public ApplicationSettingsProvider() {
			_config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
		}

		public int WebUiPort {
			get {
				return int.Parse(GetValue("webUiPort"));
			}
			set {
				SetValue("webUiPort", value);
			}
		}

		public string WebUiPath {
			get {
				return GetValue("webUiPath");
			}
			set {
				SetValue("webUiPath", value);
			}
		}

		public bool SettingExists(string key) {
			return _config.AppSettings.Settings[key] != null;
		}

		public string GetValue(string key, string defaultValue = null) {
			if (SettingExists(key))
				return _config.AppSettings.Settings[key].Value;
			if (defaultValue == null)
				throw new SettingNotFoundException(key);
			return defaultValue;
		}

		public void SetValue(string key, object value) {
			var valueString = value.ToString();
			if (!SettingExists(key))
				_config.AppSettings.Settings.Add(key, valueString);
			else
				_config.AppSettings.Settings[key].Value = valueString;
		}

		public void UpdateSettings(ISettingsData settingsData) {
			WebUiPort = settingsData.WebUiPort;
			WebUiPath = settingsData.WebUiPath;
		}

		public void Save() {
			_config.Save(ConfigurationSaveMode.Modified);
		}
	}
}