﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Loader;
using HAServer.Interfaces.Core;

namespace HAServer.Core {
	public class ComponentLoader<T> {
		private const string SearchPattern = "*.dll";
		private readonly string _componentDirectory;
		private readonly IDependencyResolver _dependencyResolver;
		private readonly List<T> _components=new List<T>();
		private readonly Type _interface;

		static ComponentLoader() {
			AppDomain.CurrentDomain.AssemblyResolve += CurrentDomain_AssemblyResolve;
		}

		public ComponentLoader(string componentDirectory, IDependencyResolver dependencyResolver) {
			_componentDirectory = componentDirectory;
			_dependencyResolver = dependencyResolver;
			_interface = typeof(T);
		}

		public IEnumerable<T> Components => _components;

		public void Load() {
			if (!Directory.Exists(_componentDirectory))
				return;

			var files = Directory.GetFiles(_componentDirectory, SearchPattern).Select(Path.GetFullPath);
			var assemblies = files.Select(AssemblyLoadContext.Default.LoadFromAssemblyPath).ToList();
			var types = assemblies.SelectMany(assembly => assembly.GetExportedTypes());
			foreach (var type in types) {
				if (type.IsClass && _interface.IsAssignableFrom(type)) {
					_components.Add((T)_dependencyResolver.CreateInstance(type));
				}
			}
		}

		private static Assembly CurrentDomain_AssemblyResolve(object sender, ResolveEventArgs args) {
			var asm = AppDomain.CurrentDomain.GetAssemblies().FirstOrDefault(assembly => assembly.FullName==args.Name);
			if (asm == null)
				throw new AssemblyNotFoundException(args.Name);
			return asm;
		}
	}
}
