﻿using System;

namespace HAServer.Core {
	public class Condition {
		private const double Epsilon = 1E-15;

		public Comparator Comparator { get; }
		public string Topic { get; }
		public virtual string Value { get; }

		protected Condition(Comparator comparator, string topic)
		{
			Comparator = comparator;
			Topic = topic;
		}

		public Condition(Comparator comparator, string topic, string value):this(comparator, topic) {
			Value = value;
		}

		public bool IsTrue(string inputValue) {
			double inputNumeric, outputNumeric;
			if (double.TryParse(inputValue, out inputNumeric) && double.TryParse(Value, out outputNumeric))
				return CompareNumeric(inputNumeric, outputNumeric);

			TimeSpan inputTime, outputTime;
			if (TimeSpan.TryParse(inputValue, out inputTime) && TimeSpan.TryParse(Value, out outputTime))
				return CompareNumeric(inputTime.TotalSeconds, outputTime.TotalSeconds);

			switch (Comparator) {
				case Comparator.Equal:
					return inputValue == Value;
				case Comparator.NotEqual:
					return inputValue != Value;
			}

			return false;
		}

		private bool CompareNumeric(double input, double value) {
			switch (Comparator) {
				case Comparator.Equal:
					return Math.Abs(input - value) < Epsilon;
				case Comparator.NotEqual:
					return Math.Abs(input - value) > Epsilon;
				case Comparator.LessThan:
					return input < value;
				case Comparator.LessThanOrEqual:
					return input <= value;
				case Comparator.GreaterThan:
					return input > value;
				case Comparator.GreaterThanOrEqual:
					return input >= value;
				default:
					throw new ArgumentOutOfRangeException();
			}
		}
	}
}