﻿using System;
using System.Collections.Generic;
using System.Linq;
using HAServer.Interfaces.Core;

namespace HAServer.Core {
	public class DependencyResolver:IDependencyResolver {
		private readonly Dictionary<Type, object> _dependencies=new Dictionary<Type, object>();

		public void Register<T>(T value) {
			var type = typeof(T);
			if (_dependencies.ContainsKey(type))
				throw new Exception("Type has already been registered");
			_dependencies[type] = value;
		}

		public T Resolve<T>() {
			var type = typeof(T);
			if (!_dependencies.ContainsKey(type))
				throw new Exception("Type has not been registered");
			return (T)_dependencies[type];
		}

		public object CreateInstance(Type type) {
			var constructors = type.GetConstructors().OrderByDescending(constructor => constructor.GetParameters().Length);
			foreach (var constructor in constructors) {
				var parameters = constructor.GetParameters();
				if (parameters.All(parameter=>_dependencies.ContainsKey(parameter.ParameterType)))
					return Activator.CreateInstance(type, parameters.Select(info => _dependencies[info.ParameterType]).ToArray());
			}
			throw new Exception("Could not instantiate object: no dependencies registered for constructor");
		}

		public T CreateInstance<T>() {
			return (T)CreateInstance(typeof(T));
		}
	}
}