﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HAServer.Devices;

namespace HAServer.Core {
	public class DeviceGroup:ICollection<BaseDevice> {
		private readonly List<BaseDevice> _devices=new List<BaseDevice>();

		public DeviceGroup(string name) {
			Name = name;
		}

		public BaseDevice this[int index] => _devices[index];

		public string Name { get; }

		public void Add(BaseDevice device) {
			_devices.Add(device);
		}

		public int Count => _devices.Count;
		public bool IsReadOnly => false;

		public void TurnOn() {
			var switches = _devices.OfType<Switch>();
			foreach (var device in switches) {
				device.TurnOn();
			}
		}

		public void TurnOff() {
			var switches = _devices.OfType<Switch>();
			foreach (var device in switches) {
				device.TurnOff();
			}
		}

		public void Clear() {
			_devices.Clear();
		}

		public bool Contains(BaseDevice item) {
			return _devices.Contains(item);
		}

		public void CopyTo(BaseDevice[] array, int arrayIndex) {
			_devices.CopyTo(array, arrayIndex);
		}

		public bool Remove(BaseDevice item) {
			return _devices.Remove(item);
		}

		public IEnumerator<BaseDevice> GetEnumerator() {
			return _devices.GetEnumerator();
		}

		IEnumerator IEnumerable.GetEnumerator() {
			return GetEnumerator();
		}
	}
}
