﻿using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using HAServer.Interfaces.Core;

namespace HAServer.Core {
	public class MessageBus:IMessageBus {
		private readonly Dictionary<string, List<ISubscriber>> _subscribers = new Dictionary<string, List<ISubscriber>>();
		private readonly ILogProvider _logProvider;

		public MessageBus(ILogProvider logProvider) {
			_logProvider = logProvider;
		}

		public void AddSubscriber(ISubscriber subscriber, string topic) {
			if (!_subscribers.ContainsKey(topic))
				_subscribers[topic] = new List<ISubscriber>();

			_subscribers[topic].Add(subscriber);
		}

		public IEnumerable<ISubscriber> GetSubscribers(string topic) {
			return _subscribers.Where(pair => Regex.Match(topic, pair.Key).Success).SelectMany(pair => pair.Value);
		}

		public void RemoveSubscriber(ISubscriber subscriber) {
			var keysToRemove = new List<string>();
			foreach (var items in _subscribers) {
				_subscribers[items.Key].Remove(subscriber);
				if (_subscribers[items.Key].Count == 0)
					keysToRemove.Add(items.Key);
			}

			foreach (var key in keysToRemove) {
				_subscribers.Remove(key);
			}
		}

		public void PublishMessage(string topic, string message, ISubscriber publisher) {
			_logProvider.Write($"Publishing '{message}' to '{topic}'", LogLevel.Debug);

			var subscribers = GetSubscribers(topic);

			foreach (var subscriber in subscribers) {
				if (subscriber != publisher)
					subscriber.ProcessMessage(topic, message);
			}
		}
	}
}
