﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using HAServer.Interfaces.Core;
using HAServer.Serialisation;
using Newtonsoft.Json;

namespace HAServer.Core {
	public class TriggerConfig {
		private List<Trigger> _triggers = new List<Trigger>();
		private string _filename = "triggers.json";
		private int _triggerId;

		public static TriggerConfig Load(string filename, IMessageBus messageBus, DeviceConfig deviceConfig, ILogProvider logProvider) {
			var settings = new JsonSerialiserSettings(messageBus, deviceConfig, logProvider);
			var triggers = JsonConvert.DeserializeObject<TriggerConfig>(File.ReadAllText(filename), settings);
			triggers._triggers = triggers._triggers.Where(trigger => trigger != null).ToList();
			triggers._filename = filename;
			return triggers;
		}

		private int GetId() => _triggerId++;

		public void Add(Trigger trigger) {
			trigger.Id = GetId();
			_triggers.Add(trigger);
		}

		public void Save(string filename = null) {
			File.WriteAllText(filename ?? _filename, JsonConvert.SerializeObject(this, Formatting.Indented));
		}

		public IEnumerable<Trigger> Triggers => _triggers;

		public void Remove(int triggerId) {
			var trigger = _triggers.Find(t => t.Id == triggerId);
			_triggers.Remove(trigger);
			trigger.Dispose();
		}
	}
}
