﻿using HAServer.Interfaces.Core;

namespace HAServer.Devices {
	public abstract class BaseDevice: ISubscriber {
		public const string DeviceUpdateTopic= "updated_device";

		protected readonly IMessageBus MessageBus;
		private string _value;

		protected BaseDevice(IMessageBus messageBus, string topic, string name) {
			MessageBus = messageBus;
			Topic = topic;
			Name = name;

			MessageBus.AddSubscriber(this, $"^{Topic}$");
		}

		public virtual void ProcessMessage(string topic, string message) {
			Value = message;
		}

		public string Value {
			get {
				return _value;
			}
			protected set {
				_value = value;
				PublishUpdate();
			}
		}

		private void PublishUpdate() {
			MessageBus.PublishMessage(DeviceUpdateTopic, Topic, this);
		}

		public string Topic { get; }
		public string Name { get; }
		public string Type => GetType().Name;
	}
}
