using System;
using System.Collections.Generic;
using HAServer.Devices;
using HAServer.Interfaces.Core;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace HAServer.Serialisation {
	public class BaseDeviceJsonConverter:JsonConverter<BaseDevice> {
		private readonly IMessageBus _messageBus;
		private readonly Dictionary<string, Func<JObject, BaseDevice>> _typeCreationMap;

		public BaseDeviceJsonConverter(IMessageBus messageBus) {
			_messageBus = messageBus;
			_typeCreationMap = new Dictionary<string, Func<JObject, BaseDevice>> {
				{typeof(Switch).Name, CreateSwitch},
				{typeof(Sensor).Name, CreateSensor}
			};
		}

		public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer) {
			var jsonObject = JObject.Load(reader);
			var typeName = jsonObject["Type"].Value<string>();
			return _typeCreationMap[typeName](jsonObject);
		}

		public override bool CanWrite => false;

		public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer) {
			throw new NotImplementedException();
		}

		private Sensor CreateSensor(JObject jsonObject) {
			return new Sensor(_messageBus, jsonObject["Topic"].Value<string>(), jsonObject["Name"].Value<string>(), jsonObject["Unit"].Value<string>());
		}

		private Switch CreateSwitch(JObject jsonObject) {
			return new Switch(_messageBus,
							  jsonObject["Topic"].Value<string>(),
							  jsonObject["Name"].Value<string>(),
							  jsonObject["OnValue"].Value<string>(),
							  jsonObject["OffValue"].Value<string>());
		}
	}
}