﻿using System;
using HAServer.Core;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace HAServer.Serialisation {
	public class TriggerActionJsonConverter:JsonConverter<TriggerAction> {
		private readonly DeviceConfig _deviceConfig;

		public TriggerActionJsonConverter(DeviceConfig deviceConfig) {
			_deviceConfig = deviceConfig;
		}

		public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer) {
			throw new NotImplementedException();
		}

		public override bool CanWrite => false;

		public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer) {
			var jsonObject = JObject.Load(reader);
			var type = jsonObject["Type"].ToObject<TriggerActionType>();
			var identifier = jsonObject["Identifier"].ToString();
			var method = jsonObject["Method"].ToString();

			Action action;
			switch (type) {
				case TriggerActionType.Device:
					if (!_deviceConfig.Devices.ContainsKey(identifier))
						return null;

					var device = _deviceConfig.Devices[identifier];
					var deviceMethod = device.GetType().GetMethod(method);
					action = (Action)Delegate.CreateDelegate(typeof(Action), device, deviceMethod);
					break;
				case TriggerActionType.Group:
					var group = _deviceConfig.Groups[identifier];
					var groupMethod = group.GetType().GetMethod(method);
					action = (Action)Delegate.CreateDelegate(typeof(Action), group, groupMethod);
					break;
				default:
					throw new ArgumentOutOfRangeException();
			}

			return new TriggerAction(type, identifier, action);
		}
	}
}
