﻿using System;
using HAServer.Core;
using HAServer.Interfaces.Core;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace HAServer.Serialisation {
	public class TriggerJsonConverter:JsonConverter<Trigger> {
		private readonly IMessageBus _messageBus;
		private readonly ILogProvider _logProvider;

		public TriggerJsonConverter(IMessageBus messageBus, ILogProvider logProvider) {
			_messageBus = messageBus;
			_logProvider = logProvider;
		}

		public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer) {
			throw new NotImplementedException();
		}

		public override bool CanWrite => false;

		public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer) {
			var jsonObject = JObject.Load(reader);
			var name = jsonObject["Name"].Value<string>();
			var triggerType = jsonObject["TriggerType"].Value<int>();
			var enabled = jsonObject["Enabled"]?.Value<bool>() ?? true;
			var conditions = serializer.Deserialize<Condition[]>(jsonObject["Conditions"].CreateReader());
			var actions = serializer.Deserialize<TriggerAction[]>(jsonObject["Actions"].CreateReader());

			return new Trigger(_messageBus, name, (TriggerType)triggerType, actions, conditions, _logProvider) {
				Enabled = enabled
			};
		}
	}
}
