﻿using System.Linq;
using HAServer.Core;
using HAServer.Devices;
using HAServer.WebServer.DTO;
using HAServer.WebServer.RequestHandlers;

namespace HAServer.WebServer {
	public class ApiMethods {
		private readonly WebSocketApi _webSocketApi;
		private readonly DeviceConfig _deviceConfig;
		private readonly TriggerConfig _triggerConfig;
		private readonly IApplicationSettingsProvider _settingsProvider;
		private readonly WebObjectHandler _webObjectHandler;

		public ApiMethods(WebSocketApi webSocketApi, DeviceConfig deviceConfig, TriggerConfig triggerConfig, IApplicationSettingsProvider settingsProvider, WebObjectHandler webObjectHandler) {
			_webSocketApi = webSocketApi;
			_deviceConfig = deviceConfig;
			_triggerConfig = triggerConfig;
			_settingsProvider = settingsProvider;
			_webObjectHandler = webObjectHandler;
		}

		public void SetSwitchValue(string topic, bool on) {
			if (!_deviceConfig.Devices.ContainsKey(topic) ||!(_deviceConfig.Devices[topic] is Switch))
				return;

			((Switch)_deviceConfig.Devices[topic]).On = on;
		}

		public void AddDevice(BaseDevice device) {
			_deviceConfig.AddDevice(device);
			_deviceConfig.Save();
			_webSocketApi.UpdateDevices(true);
		}

		public void DeleteDevice(string topic) {
			_deviceConfig.RemoveDevice(topic);
			_deviceConfig.Save();
			_webSocketApi.UpdateDevices(true);
		}

		public void AddGroup(string groupName) {
			_deviceConfig.AddGroup(groupName);
			_deviceConfig.Save();
			_webSocketApi.UpdateGroups(true);
		}

		public void DeleteGroup(string groupName) {
			_deviceConfig.RemoveGroup(groupName);
			_deviceConfig.Save();
			_webSocketApi.UpdateGroups(true);
		}

		public void AddDeviceToGroup(string groupName, string deviceTopic) {
			_deviceConfig.AddDeviceToGroup(groupName, deviceTopic);
			_deviceConfig.Save();
			_webSocketApi.UpdateGroups(true);
		}

		public void RemoveDeviceFromGroup(string groupName, string deviceTopic) {
			_deviceConfig.RemoveDeviceFromGroup(groupName, deviceTopic);
			_deviceConfig.Save();
			_webSocketApi.UpdateGroups(true);
		}

		public void AddTrigger(Trigger trigger) {
			_triggerConfig.Add(trigger);
			_triggerConfig.Save();
			_webSocketApi.UpdateTriggers();
		}

		public void ReloadTriggers() {
			_webSocketApi.UpdateTriggers();
		}

		public void DeleteTrigger(long triggerId) {
			_triggerConfig.Remove((int)triggerId);
			_triggerConfig.Save();
			_webSocketApi.UpdateTriggers();
		}

		public void EnableTrigger(long triggerId, bool enable) {
			var trigger = _triggerConfig.Triggers.First(t => t.Id == triggerId);
			trigger.Enabled = enable;
			_triggerConfig.Save();
		}

		public void SaveSettings(SettingsDto settingsData) {
			_settingsProvider.UpdateSettings(settingsData);
			_settingsProvider.Save();
			_webSocketApi.UpdateSettings();
		}

		public void TurnOnGroup(string groupName) {
			_deviceConfig.Groups[groupName].TurnOn();
		}

		public void TurnOffGroup(string groupName) {
			_deviceConfig.Groups[groupName].TurnOff();
		}

		public void SaveObject(string filename, string data) {
			_webObjectHandler.SaveObject(filename, data);
		}
	}
}
