﻿using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Reflection;
using HAServer.Devices;

namespace HAServer.WebServer.DTO {
	[ExcludeFromCodeCoverage]
	public class DeviceDto {
		public DeviceDto(BaseDevice device) {
			if (device is Switch)
				Value = ((Switch)device).On;
			else
				Value = device.Value;

			if (device is Sensor)
				Unit = ((Sensor)device).Unit;

			Topic = device.Topic;
			Name = device.Name;
			Type = device.Type;

			Methods = device.GetType()
							.GetMethods(BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly)
							.Where(method => !method.IsSpecialName)
							.Select(method => method.Name);
		}

		public string Name { get; }
		public object Value { get; }
		public string Unit { get; }
		public string Topic { get; }
		public string Type { get; }
		public IEnumerable<string> Methods { get; }
	}
}
