﻿app.controller("triggers", ["$scope", "deviceService", "triggerService", function ($scope, deviceService, triggerService) {
	$scope.setTitle("Triggers");
	$scope.devices = deviceService.devices;
	$scope.groups = deviceService.groups;
	triggerService.ReloadTriggers();
	setupActionables();

	$scope.comparators = {
		0: "Equal",
		1: "Not Equal",
		2: "Less Than",
		3: "Less Than Or Equal",
		4: "Greater Than",
		5: "Greater Than Or Equal"
	};

	$scope.actionTypes = {
		0: "Devices",
		1: "Groups"
	};

	Object.defineProperty($scope, "triggers", {
		get: function () {
			return triggerService.triggers;
		}
	});

	$scope.$watch("devices", setupActionables);
	$scope.$watch("groups", setupActionables);

	function setupActionables() {
		$scope.actionables = [];
		angular.forEach($scope.devices, function (device) {
			$scope.actionables.push({ Type: 0, Identifier: device.Topic });
		});
		angular.forEach($scope.groups, function (group, groupName) {
			$scope.actionables.push({ Type: 1, Identifier: groupName });
		});
	}

	$scope.resolveTopicName = function (topic) {
		if ($scope.devices[topic])
			return $scope.devices[topic].Name;
		return topic;
	};

	$scope.resolveActionName = function (action) {
		switch (action.Type) {
			case 0:
				return $scope.resolveTopicName(action.Identifier);
			case 1:
				return action.Identifier;
		}
		return action.Identifier;
	};

	$scope.resolveMethods = function (action) {
		switch (action.Type) {
			case 0:
				if ($scope.devices[action.Identifier])
					return $scope.devices[action.Identifier].Methods;
				break;
			case 1:
				return ["TurnOn", "TurnOff"];
		}
	};

	function resetNewTrigger() {
		$scope.newTrigger = {
			Name: "",
			TriggerType: "",
			DeviceTopic: "",
			DeviceMethod: "",
			Conditions: [],
			Actions: []
		};
		resetNewCondition();
		resetNewAction();
	}

	function resetNewCondition() {
		$scope.newCondition = {
			Topic: "",
			Comparator: "",
			Value: ""
		};
	}

	function resetNewAction() {
		$scope.newAction = {
			Type: 0,
			Identifier: "",
			Method: ""
		};
	}

	$scope.addCondition = function (newCondition) {
		var condition = {
			Topic: newCondition.Topic,
			Comparator: newCondition.Comparator
		};

		if (newCondition.IsDevice)
			condition.Device = newCondition.Value;
		else
			condition.Value = newCondition.Value;

		$scope.newTrigger.Conditions.push(condition);
		resetNewCondition();
	};

	$scope.removeCondition = function (condition) {
		$scope.newTrigger.Conditions = $scope.newTrigger.Conditions.filter(function (element) {
			return element !== condition;
		});
	};

	$scope.addAction = function (action) {
		$scope.newTrigger.Actions.push(angular.copy(action));
		console.log($scope.newTrigger);
	};

	$scope.removeAction = function (action) {
		$scope.newTrigger.Actions = $scope.newTrigger.Actions.filter(function (element) {
			return element !== action;
		});
	};

	$scope.saveTrigger = function () {
		triggerService.AddTrigger($scope.newTrigger);
	};

	$scope.deleteTrigger = function (trigger) {
		triggerService.DeleteTrigger(trigger);
	};

	resetNewTrigger();
}]);
