﻿app.service("deviceService", ["webApiService", "objectService", function (webApiService, objectService) {
	var groupsOrderFilename = "groupsOrder.json";
	var self = this;
	self.devices = {};
	self.groups = {};
	self.groupsOrder = {};

	webApiService.registerCallbackService(self);

	objectService.Load(groupsOrderFilename).then(function (order) {
		self.groupsOrder = order;
		self.UpdateGroups(angular.copy(self.groups));
	});

	var deviceFactory = {
		switch: function (device) {
			Object.defineProperty(device, "On", {
				get: function () {
					return device.Value;
				},
				set: function (newValue) {
					device.Value = newValue;
					webApiService.SetSwitchValue(device.Topic, device.Value);
				}
			});
			return device;
		},
		sensor: function (device) {
			return device;
		}
	};

	function clearObject(object) {
		for (var member in object)
			delete object[member];
	}

	self.UpdateDevice = function (device) {
		self.devices[device.Topic].Value = device.Value;
	};

	function cleanUpDevices() {
		angular.forEach(self.groups, function (group, key) {
			angular.forEach(group, function (topic, index) {
				if (angular.isUndefined(self.devices[topic])) {
					self.groups[key].splice(index, 1);
				}
			});
		});
	}

	self.UpdateDevices = function (devices) {
		clearObject(self.devices);
		angular.forEach(devices, function (device) {
			var typeIndex = device.Type.toLowerCase();
			var topic = device.Topic;
			self.devices[topic] = deviceFactory[typeIndex](device);
		});
		cleanUpDevices();
	};

	self.UpdateGroups = function (groups) {
		clearObject(self.groups);

		var orderedGroups = [];
		var remainingGroups = [];
		for (var property in groups) {
			if (groups.hasOwnProperty(property)) {
				var group = {
					Name: property,
					Devices: groups[property]
				};

				if (self.groupsOrder[group.Name])
					orderedGroups[self.groupsOrder[group.Name]] = group;
				else
					remainingGroups.push(group);
			}
		}

		angular.forEach(orderedGroups, function (group) {
			self.groups[group.Name] = group.Devices;
		});
		angular.forEach(remainingGroups, function (group) {
			self.groups[group.Name] = group.Devices;
		});
	};

	self.AddDevice = function (newDevice) {
		webApiService.AddDevice(newDevice);
	};

	self.DeleteDevice = function (device) {
		webApiService.DeleteDevice(device.Topic);
	};

	self.AddGroup = function (groupName) {
		webApiService.AddGroup(groupName);
	};

	self.DeleteGroup = function (groupName) {
		webApiService.DeleteGroup(groupName);
	};

	function resetGroupOrderIndex() {
		var index = 0;
		for (var property in self.groups) {
			if (self.groups.hasOwnProperty(property)) {
				self.groupsOrder[property] = ++index;
			}
		}
	}

	function getIndexedGroupNames() {
		var indexedGroups = [];
		for (var property in self.groupsOrder) {
			if (self.groupsOrder.hasOwnProperty(property)) {
				indexedGroups[self.groupsOrder[property]] = property;
			}
		}
		return indexedGroups;
	}

	function setGroupOrderFromIndexedArray(indexedGroups) {
		clearObject(self.groupsOrder);
		for (var i = 0; i < indexedGroups.length; i++) {
			self.groupsOrder[indexedGroups[i]] = i;
		}

		self.UpdateGroups(angular.copy(self.groups));
	}

	function moveGroup(groupName, direction) {
		resetGroupOrderIndex();

		var indexedGroups = getIndexedGroupNames();
		var currentIndex = self.groupsOrder[groupName];
		var indexToSwap = currentIndex + direction;

		indexedGroups[currentIndex] = indexedGroups[indexToSwap];
		indexedGroups[indexToSwap] = groupName;

		setGroupOrderFromIndexedArray(indexedGroups);

		webApiService.SaveObject(groupsOrderFilename, self.groupsOrder);
	}

	self.MoveGroupUp = function (groupName) {
		moveGroup(groupName, -1);
	};

	self.MoveGroupDown = function (groupName) {
		moveGroup(groupName, 1);
	};

	self.AddDeviceToGroup = function (group, device) {
		webApiService.AddDeviceToGroup(group, device.Topic);
	};

	self.RemoveDeviceFromGroup = function (group, device) {
		webApiService.RemoveDeviceFromGroup(group, device.Topic);
	};

	self.TurnOnGroup = function (groupName) {
		webApiService.TurnOnGroup(groupName);
	};

	self.TurnOffGroup = function (groupName) {
		webApiService.TurnOffGroup(groupName);
	};
}]);