﻿var WebApiCallbackEventType = Object.freeze({
	Open: 0,
	Message: 1,
	Close: 2
});

app.service("webApiService", [function () {
	var self = this;
	var location = window.location;
	var socket;
	var messageCallbacks = [];
	var services = [];

	function doCallbacks(callbackType, data) {
		angular.forEach(messageCallbacks, function (callback) {
			var event = {
				type: callbackType,
				data: data
			};
			callback(event);
		});
	}

	function socketOpen(event) {
		doCallbacks(WebApiCallbackEventType.Open, event);
	}

	function socketMessage(event) {
		var remoteProcedureCall = JSON.parse(event.data);
		var method = remoteProcedureCall.Method;
		var parameters = remoteProcedureCall.Parameters;
		angular.forEach(services, function (service) {
			if (typeof service[method] === "function")
				service[method](...parameters);
		});

		doCallbacks(WebApiCallbackEventType.Message);
	}

	function socketClose(event) {
		doCallbacks(WebApiCallbackEventType.Close, event);
		if (!event.wasClean)
			setTimeout(startSocket, 500);
	}

	function send(rpc) {
		socket.send(JSON.stringify(rpc));
	}

	function startSocket() {
		socket = new WebSocket((location.protocol === "https:" ? "wss" : "ws") + "://" + location.host + "/api");
		socket.onopen = socketOpen;
		socket.onmessage = socketMessage;
		socket.onclose = socketClose;
	}

	startSocket();

	self.registerMesssageCallback = function (callback) {
		messageCallbacks.push(callback);
	};

	self.registerCallbackService = function (service) {
		services.push(service);
	};

	self.SetSwitchValue = function (topic, status) {
		var rpc = {
			Method: "SetSwitchValue",
			Parameters: [topic, status]
		};
		send(rpc);
	};

	self.AddDevice = function (newDevice) {
		var rpc = {
			Method: "AddDevice",
			Parameters: [newDevice]
		};
		send(rpc);
	};

	self.DeleteDevice = function (topic) {
		var rpc = {
			Method: "DeleteDevice",
			Parameters: [topic]
		};
		send(rpc);
	};

	self.AddGroup = function (groupName) {
		var rpc = {
			Method: "AddGroup",
			Parameters: [groupName]
		};
		send(rpc);
	};

	self.DeleteGroup = function (groupName) {
		var rpc = {
			Method: "DeleteGroup",
			Parameters: [groupName]
		};
		send(rpc);
	};

	self.AddDeviceToGroup = function (groupName, device) {
		var rpc = {
			Method: "AddDeviceToGroup",
			Parameters: [groupName, device]
		};
		send(rpc);
	};

	self.RemoveDeviceFromGroup = function (groupName, device) {
		var rpc = {
			Method: "RemoveDeviceFromGroup",
			Parameters: [groupName, device]
		};
		send(rpc);
	};

	self.AddTrigger = function(trigger) {
		var rpc = {
			Method: "AddTrigger",
			Parameters: [trigger]
		};
		send(rpc);
	};

	self.DeleteTrigger = function(triggerId) {
		var rpc = {
			Method: "DeleteTrigger",
			Parameters: [triggerId]
		};
		send(rpc);
	};

	self.EnableTrigger = function(triggerId, enable) {
		var rpc = {
			Method: "EnableTrigger",
			Parameters: [triggerId, enable]
		};
		send(rpc);
	};

	self.ReloadTriggers = function() {
		var rpc = {
			Method: "ReloadTriggers",
			Parameters: []
		};
		send(rpc);
	};

	self.SaveSettings = function (settings) {
		var rpc = {
			Method: "SaveSettings",
			Parameters: [settings]
		};
		send(rpc);
	};

	self.TurnOnGroup = function (groupName) {
		var rpc = {
			Method: "TurnOnGroup",
			Parameters: [groupName]
		};
		send(rpc);
	};

	self.TurnOffGroup = function (groupName) {
		var rpc = {
			Method: "TurnOffGroup",
			Parameters: [groupName]
		};
		send(rpc);
	};

	self.SaveObject = function(filename, object) {
		var rpc = {
			Method: "SaveObject",
			Parameters: [filename, JSON.stringify(object)]
		};
		send(rpc);
	};
}]);