import React from 'react';
import Menu from './Components/Menu/menu';
import Loading from './Components/Loading/loading';
import SocketError from './Components/SocketError/socketError';
import Routing from './routing';
import './styles/App.scss'
import { BrowserRouter as Router, Link } from "react-router-dom";

export default class App extends React.Component {
	state = {
		menuOpen: false,
		loading: false,
		socketError: false
	};

	menu = React.createRef();

	toggleMenu() {
		if (this.state.loading)
			return;

		this.menu.current.toggle();
	}

	render() {
		return (
			<Router>
				<header>
					<button className="menu imageButton" onClick={this.toggleMenu.bind(this)}><img src="/images/menu.svg" alt="menu" /></button>
					<Link className="heading" to="/">
						<h1>{document.title}</h1>
					</Link>
				</header>
				<main>
					<section className="content">
						<Routing groups={this.groups} devices={this.devices} />
					</section>
					<Loading show={this.state.loading} />
					<SocketError show={this.state.socketError} />
					<Menu ref={this.menu} />
				</main>
			</Router>
		);
	}
}