import React from 'react';

export default class Devices extends React.Component {
	state = {
		newDevice: {
			name: "",
			topic: "",
			onValue: "",
			offValue: "",
			unit: "",
			type: "",
			valid: false
		},
		newGroupName: "",
		newDeviceGroup: {
			group: "",
			device: "",
			valid: false
		}
	}

	updateNewDevice(property, value){
		const device = this.state.newDevice;
		device[property] = value;
		device.valid = device.name !== ""
			&& device.topic !== ""
			&& device.type !== ""
			&& (
				(device.type === "Sensor" && device.unit !== "")
				|| (device.type === "Switch" && device.onValue !== "" && device.offValue !== "")
			);
		this.setState({ newDevice: device});
	}

	updateGroupName(name) {
		this.setState({ newGroupName: name });
	}

	addDevice() {
		console.log(this.state.newDevice);
	}

	deleteDevice(topic) {
		console.log(topic);
	}

	addGroup() {
		console.log(this.state.newGroupName);
	}

	moveGroupUp(group) {
		console.log(group);
	}

	moveGroupDown(group) {
		console.log(group);
	}

	deleteGroup(group) {
		console.log(group);
	}

	removeDeviceFromGroup(group, device) {
		console.log(group, device);
	}

	updateNewDeviceGroup(property, value) {
		const deviceGroup = this.state.newDeviceGroup;
		deviceGroup[property] = value;
		deviceGroup.valid = deviceGroup.group !== "" && deviceGroup.device !== "";
		this.setState({ newDeviceGroup: deviceGroup });
	}

	addDeviceToGroup() {
		console.log(this.state.newDeviceGroup);
	}

	componentDidMount() {
		document.title = "Devices";
	}

	renderDevices(devices) {
		return Object.keys(devices).map((key, index) => <div className="device" key={key}>
			<span className="deviceName">{devices[key].Name}</span>
			{(function () {
				switch (devices[key].Type) {
					case "Switch":
						return <img src="/images/switch.svg" alt="Switch" />
					case "Sensor":
						return <span>{devices[key].Unit}</span>
					default:
						return <span>?</span>
				}
			})()}
			<button className="imageButton" onClick={() => this.deleteDevice(key)}>
				<img src="/images/delete.svg" alt="Delete" />
			</button>
		</div>);
	}

	renderNewDeviceInputs() {
		return <div className="cardRow">
			<input placeholder="Name" type="text" required onChange={(e) => this.updateNewDevice("name", e.target.value)} />
			<select required defaultValue="" onChange={(e) => this.updateNewDevice("type", e.target.value)}>
				<option disabled="disabled" value="">Select type</option>
				<option value="Switch">Switch</option>
				<option value="Sensor">Sensor</option>
			</select>
			<input placeholder="Topic" type="text" required onChange={(e) => this.updateNewDevice("topic", e.target.value)} />
			{this.state.newDevice.type === "Switch" && <input placeholder="On value" type="text" required onChange={(e) => this.updateNewDevice("onValue", e.target.value)} />}
			{this.state.newDevice.type === "Switch" && <input placeholder="Off value" type="text" required onChange={(e) => this.updateNewDevice("offValue", e.target.value)} />}
			{this.state.newDevice.type === "Sensor" && <select defaultValue="" onChange={(e) => this.updateNewDevice("unit", e.target.value)}>
				<option value="" disabled="disabled">Unit</option>
				<option>%</option>
				<option>&deg;C</option>
			</select>}
			<button className="imageButton" disabled={!this.state.newDevice.valid} onClick={()=>this.addDevice()}>
				<img src="/images/add.svg" alt="Add Device" />
			</button>
		</div>
	}

	renderGroups(groups, devices) {
		return groups.map((group, index) => <div className="cardRow groupRow" key={index}>
			<div className="headingWithActions">
				<h3 className="groupName">{group[0]}</h3>
				<div className="actions">
					<button className="imageButton" onClick={() => this.moveGroupUp(group[0])}>
						<img src="/images/up.svg" alt="Move up" />
					</button>
					<button className="imageButton" onClick={() => this.moveGroupDown(group[0])}>
						<img src="/images/down.svg" alt="Move down" />
					</button>
					<button className="imageButton" onClick={() => this.deleteGroup(group[0])}>
						<img src="/images/delete.svg" alt="Delete" />
					</button>
				</div>
			</div>
			{group[1].map((topic, index) => <div className="device" key={index}>
				<span className="deviceName">{devices[topic].Name}</span>
				<button className="imageButton" onClick={() => this.removeDeviceFromGroup(group[0], topic)}>
					<img src="/images/delete.svg" alt="Delete" />
				</button>
			</div>)}
		</div>);
	}

	renderGroupAssignmentInputs(groups, devices) {
		return <>
			<hr />
			<h2>Add Device to Group</h2>
			<div className="cardRow">
				<select required defaultValue="" onChange={(e) => this.updateNewDeviceGroup("group", e.target.value)}>
					<option disabled="disabled" value="">Select group</option>
					{groups.map((group, index) => <option key={index}>{group[0]}</option>)}
				</select>
				<select required defaultValue="" onChange={(e) => this.updateNewDeviceGroup("device", e.target.value)}>
					<option disabled="disabled" value="">Select device</option>
					{Object.keys(devices).map((topic, index) => <option key={topic} value={topic}>{devices[topic].Name}</option>)}
				</select>
				<button className="imageButton" disabled={!this.state.newDeviceGroup.valid} onClick={() => this.addDeviceToGroup()}>
					<img src="/images/add.svg" alt="Add to Group" />
				</button>
			</div>
		</>
	}

	renderNewGroupInputs() {
		return <>
			<h2>Add Group</h2>
			<div className="cardRow">
				<input type="text" placeholder="Group Name" required onChange={(e) => this.updateGroupName(e.target.value)} />
				<button className="imageButton" disabled={this.state.newGroupName === ""} onClick={() => this.addGroup()}>
					<img src="/images/add.svg" alt="Add Group" />
				</button>
			</div>
		</>
	}

	render() {
		const devices = this.props.devices;
		const groups = Object.entries(this.props.groups);
		const deviceCount = Object.keys(devices).length;
		return (
			<div className="cardGroup devices">
				<div className="card">
					{deviceCount > 0 && <h2>Device List</h2>}
					{deviceCount > 0 && this.renderDevices(devices)}
					{deviceCount > 0 && <hr />}
					<h2>Add new device</h2>
					{this.renderNewDeviceInputs()}
				</div>
				<div className="card">
					{groups.length > 0 && <h2>Groups</h2>}
					{groups.length > 0 && this.renderGroups(groups, devices)}
					{groups.length > 0 && this.renderGroupAssignmentInputs(groups, devices)}
					{groups.length > 0 && deviceCount > 0 && <hr />}
					{this.renderNewGroupInputs()}
				</div>
			</div>
		);
	}
}