import React from "react";
import ToggleSwitch from "../Components/ToggleSwitch/toggleSwitch";
import "./home.scss";

export default class Home extends React.Component {
	componentDidMount() {
		document.title = "Home Automation";
	}

	groupHasSwitches(groupName) {
		for (const topic of this.props.groups[groupName]) {
			if (this.props.devices[topic].Type === "Switch")
				return true;
		}
		return false;
	}

	renderControl(device) {
		switch (device.Type) {
			case "Switch":
				return <ToggleSwitch on={device.Value}></ToggleSwitch>
			case "Sensor":
				return <span>{device.Value || '--'}{device.Unit}</span>
			default:
				return <span>Unknown type</span>
		}
	}

	renderDevice(device, key) {
		return <div className="device" key={key}>
			<span className="deviceName">{device.Name}</span>
			{this.renderControl(device)}
		</div>
	}

	renderGroup(groupName, topics, devices, key) {
		return <div className="card" key={key}>
			<div className="header">
				<h2 className="title">{groupName}</h2>
				{this.groupHasSwitches(groupName) && <span className="switch">
					<a href="#" ng-click="turnOnGroup(groupName)">On</a> | <a ng-click="turnOffGroup(groupName)" href="#">Off</a>
				</span>}
			</div>
			{topics.map((topic, index) => this.renderDevice(devices[topic], index))}
		</div>
	}

	renderGroups() {
		const devices = this.props.devices;
		const groups = this.props.groups;
		var items = [];
		var key=0;
		for (const groupName in groups) {
			const topics = groups[groupName];
			if (topics.length > 0)
				items.push(this.renderGroup(groupName, topics, devices, key++));
		}
		return items;
	}

	render() {
		return (
			<div className="cardGroup">
				{this.renderGroups()}
			</div>
		);
	}
}